//
//  app.c
//
//  Auto Lock Door 0.1
//  Copyright(C)2017 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2017.02.13 v0.1  First
//    2017.02.18 v0.1a Reducing a current at power loss
//    2017.02.18 v0.1a Authentication time out
//
//

// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "peripheral/oc/plib_oc.h"
#include "music_box.h"
#include "rtc.h"
#include "system/debug/sys_debug.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;



struct settingData {
    struct mboxScore    beep1[4];
    struct mboxScore    beep2[4];
    struct mboxScore    music1[40];
    struct mboxScore    music2[121];
};

const struct settingData    settingData = {
    {
        {MBOX_SCALE_B3,  10, 1},            // Simple beep
        {0,               0, 0}             // Terminate
    },
    {
        {MBOX_SCALE_B3,  10, 1},            // Siren
        {MBOX_SCALE_B2,  10, 1},
        {0,               0, 0}
    },
    {
        {MBOX_SCALE_FS3, 13, 3},            // Swan lake
        {MBOX_SCALE_B2,   3, 1},
        {MBOX_SCALE_CS3,  3, 1},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_E3,   3, 1},
        {MBOX_SCALE_FS3,  9, 3},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_FS3,  9, 3},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_FS3,  9, 4},
        {MBOX_SCALE_B2,   3, 1},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_B2,   3, 1},
        {MBOX_SCALE_G2,   3, 1},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_B2,  10, 0},
        {MBOX_SCALE_B2,   9, 3},

        {MBOX_SCALE_E3,   3, 1},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_CS3,  3, 1},
        {MBOX_SCALE_FS3, 13, 3},

        {MBOX_SCALE_B2,   3, 1},
        {MBOX_SCALE_CS3,  3, 1},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_E3,   3, 1},
        {MBOX_SCALE_FS3,  9, 3},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_FS3,  9, 3},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_FS3,  9, 3},

        {MBOX_SCALE_B2,   3, 1},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_B2,   3, 1},
        {MBOX_SCALE_G2,   3, 1},
        {MBOX_SCALE_D3,   3, 1},
        {MBOX_SCALE_B2,  15, 0},
        {MBOX_SCALE_B2,  10, 7},
        {0,               0, 0}             // Terminate
    },
    {
        {MBOX_SCALE_D3,7,1},                // Floh walzer
        {MBOX_SCALE_C3,7,1},                //   http://www.asahi-net.or.jp/~HB9T-KTD/music/Japan/Studio/Midi/Traditional/Floh_Walzer.html
        {MBOX_SCALE_A3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_D3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A4,90,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A4,90,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A4,31,1},
        {MBOX_SCALE_C4,31,1},
        {MBOX_SCALE_D4,90,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_D4,90,1},
        {MBOX_SCALE_D4,7,1},
        {MBOX_SCALE_C4,7,1},
        {MBOX_SCALE_D4,90,1},
        {MBOX_SCALE_D4,7,1},
        {MBOX_SCALE_C4,7,1},
        {MBOX_SCALE_D4,31,1},
        {MBOX_SCALE_C4,31,1},
        {MBOX_SCALE_A4,31,1},
        {MBOX_SCALE_D3,7,17},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_F3,31,1},
        {MBOX_SCALE_C3,31,1},
        {MBOX_SCALE_F3,31,1},
        {MBOX_SCALE_C3,31,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_FS3,15,1},
        {MBOX_SCALE_G3,90,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_G3,31,1},
        {MBOX_SCALE_D3,31,1},
        {MBOX_SCALE_G3,31,1},
        {MBOX_SCALE_D3,31,1},
        {MBOX_SCALE_G3,15,1},
        {MBOX_SCALE_FS3,15,1},
        {MBOX_SCALE_G3,15,1},
        {MBOX_SCALE_GS3,15,1},
        {MBOX_SCALE_A4,90,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_A3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_D3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,7,1},
        {MBOX_SCALE_C3,7,1},
        {MBOX_SCALE_C3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_D3,15,1},
        {MBOX_SCALE_E3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {MBOX_SCALE_F3,15,1},
        {0,             0,0}             // Terminate
    },
};

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

//
//  USB host <class> call back routine
//
bool APP_UsbHostBluetoothEvent(USB_HOST_BLUETOOTH_INDEX index, USB_HOST_BLUETOOTH_EVENT event, void* pData){
    volatile int    i;

    switch(event) {
    case USB_HOST_BLUETOOTH_EVENT_ATTACH:
        appData.attach = 1;
        for(i = 0; i < 1000000; i++);
        break;
    case USB_HOST_BLUETOOTH_EVENT_DETACH:
        appData.attach = 0;
        for(i = 0; i < 1000000; i++);
        break;
    default:
        break;
    }
    return(0);
}

//
//  Bluetooth call back routines
//
void bt_attDbNotification(unsigned short handle, void *value, size_t valueLen) {

    // Essential action
    bt_att_serverNotificationQueueAdd(handle);

    // User action
    //-------------------------------------------------------------
    switch(handle) {
    case 0x1002:
        appData.lockRequest = *((unsigned char *)value);
        appData.authTimeOut = 0;
        break;
    default:
        break;
    }
    //-------------------------------------------------------------
    return;
}

long bt_securityManagerPasskeyCreateDisplay(struct bt_l2capChannel *ch) {
    appData.authTimeOut = time(NULL) + AUTH_TIME_OUT;
    appData.link        = ch->link;
    return(appData.sw1 ? 123456 : -1);
};


// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************

/******************************************************************************
  Function:
    void MOTOR_Tasks ( void )

  Remarks:
 */

typedef enum {
    MOTOR_ACTION_FREE=0,
    MOTOR_ACTION_OPEN,
    MOTOR_ACTION_CLOSE,
    MOTOR_ACTION_NONE,
} MOTOR_ACTION;

typedef enum {
    MOTOR_STATUS_INACTIVE = 0,
    MOTOR_STATUS_STABLE,
    MOTOR_STATUS_BUSY,
    MOTOR_STATUS_OVERLOAD,
} MOTOR_STATUS;

MOTOR_STATUS MOTOR_Tasks(MOTOR_ACTION action) {
    MOTOR_STATUS    stat;
    float           mc;
    static int      pwm = 0;
    static time_t   tim = 0;

    switch(action){
    case MOTOR_ACTION_FREE:
        if (pwm) {
            DRV_OC0_Stop();
            pwm = 0;
            tim = 0;
        }
        break;
    case MOTOR_ACTION_OPEN:
        if (pwm == 0) {
            DRV_OC0_Start();
        }
        if (pwm != MOTOR_OPEN_PWM) {
            pwm =  MOTOR_OPEN_PWM;
            PLIB_OC_PulseWidth16BitSet(OC_ID_2, pwm);
            tim = time(NULL);
        }
        break;
    case MOTOR_ACTION_CLOSE:
        if (pwm == 0) {
            DRV_OC0_Start();
        }
        if (pwm != MOTOR_CLOSE_PWM) {
            pwm =  MOTOR_CLOSE_PWM;
            PLIB_OC_PulseWidth16BitSet(OC_ID_2, pwm);
            tim = time(NULL);
        }
        break;
    default:
        break;
    }
    
    if (DRV_ADC_SamplesAvailable()) {
        int   a;
        float v;

        a = DRV_ADC_SamplesRead(0);
        v = (3.3/1024) * a;
        mc =  v / 1.0;    // I=V/R
    }
    
    if (tim == 0) {
        stat = MOTOR_STATUS_INACTIVE;
    } else {
        time_t  t;
        
        t = time(NULL);
        if ((t - tim) < 2) {
            stat = MOTOR_STATUS_BUSY;
        } else {
            static int  cnt;

            if (mc < MOTOR_CURRENT_LIMIT) {
                stat = MOTOR_STATUS_STABLE;
                cnt = 0;
            } else {
                if (cnt++ > 3) {
                    stat = MOTOR_STATUS_OVERLOAD;
                }
            }
        }
    }
    return(stat);
}


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;
    
    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
    // Motor PWM: Timer Driver 1 + OC0
    {
        uint32_t clk;

        appData.motorTmr = DRV_TMR_Open(DRV_TMR_INDEX_1, DRV_IO_INTENT_EXCLUSIVE);
        clk = DRV_TMR_CounterFrequencyGet(appData.motorTmr);
        DRV_TMR_AlarmRegister(appData.motorTmr, clk/50, true, 0, NULL);
        DRV_TMR_Start(appData.motorTmr);
//      DRV_OC0_Start();
        DRV_OC0_Stop();
    }

    // Motor current sense ADC
    DRV_ADC_Open();
    DRV_ADC_Start();

    // etc
    MBOX_Init();
    rtc_open();
    USB_HOST_Bluetooth_EventHandlerSet(APP_UsbHostBluetoothEvent);
    return;
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */
void APP_Tasks(void)
{
    MOTOR_ACTION    act;
    MOTOR_STATUS    r;

    /* Check the application's current state. */
    switch(appData.state){
        /* Application's initial state. */
        case APP_STATE_INIT:
            if(USB_HOST_BusEnable(0) == USB_HOST_RESULT_SUCCESS) {
                appData.state = APP_STATE_WAIT_TMR_STABLE;
            }
            break;

        /* TODO: implement your application state machine.*/
        case APP_STATE_WAIT_TMR_STABLE:
            if(USB_HOST_BusIsEnabled(0) == USB_HOST_RESULT_TRUE) {
                appData.lock        = 1;
                appData.powerState  = 0;
                appData.lockRequest = 0;
                appData.sw1         = 2;
                appData.sw2         = 2;
                appData.powerTimer  = 0;
                appData.state       = APP_STATE_IDLE;
                MBOX_Start(settingData.music1, 20, 0);
            }
            break;

        case APP_STATE_IDLE:
        {
            int     sw1;
            int     sw2;
            int     pon;
            
            // Getting a value
            sw1 = !SYS_PORTS_PinRead(PORTS_ID_0, PORT_CHANNEL_B, 13);
            sw2 = !SYS_PORTS_PinRead(PORTS_ID_0, PORT_CHANNEL_B, 15);
            pon =  SYS_PORTS_PinRead(PORTS_ID_0, PORT_CHANNEL_A, 1);

            // Action
            if (appData.powerState != pon) {
                appData.powerState  = pon;
                appData.lockRequest = 0;
                if (!pon) {
                    appData.powerTimer = 0;
                    appData.state      = APP_STATE_NO_POWER;
                }
            }
            if ((appData.authTimeOut != 0) && 
                (appData.authTimeOut < time(NULL))) {
                appData.authTimeOut = 0;
                appData.state       = APP_STATE_DISCONNECT;
            }
            if (appData.sw1 != sw1) {   // Pairing enable
                appData.sw1  = sw1;
                // see bt_securityManagerPasskeyCreateDisplay()
            }
            if (appData.sw2 != sw2) {   // Open/Close
                appData.sw2  = sw2;
                if (sw2) {
                    appData.lockRequest = !appData.lock;
                }
            }

            // Motor control
            act = MOTOR_ACTION_NONE;
            if (appData.lock != appData.lockRequest) {
                appData.lock =  appData.lockRequest;
                act = MOTOR_ACTION_OPEN;
                if (appData.lockRequest) {
                    act = MOTOR_ACTION_CLOSE;
                }
            }
            r = MOTOR_Tasks(act);
            switch(r){
            case MOTOR_STATUS_INACTIVE:
                break;
            case MOTOR_STATUS_BUSY:
                break;
            case MOTOR_STATUS_STABLE:
                break;
            case MOTOR_STATUS_OVERLOAD:
                MOTOR_Tasks(MOTOR_ACTION_FREE);
                MBOX_Start(settingData.music2, 3, 0);
                break;
            default:
                break;
            }

            // LED A (Pairing)
            if (appData.connect) {
                SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, 0);   // LED on
            } else if (appData.sw1) {
                SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, (time(NULL) & 1));    // Blink
            } else if (appData.attach) {
                static int i;
                
                if (i++ > 1000) {
                    i = 0;
                }
                SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, i < 10 ? 0 : 1);   // Flash
            } else {
                SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, 1);   // LED off
            }

            // LED B (Lock status)
            if (r == MOTOR_STATUS_STABLE) {
                if (appData.lock) {
                    SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 8, 1);   // LED off
                } else {
                    SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 8, 0);   // LED on
                }
            } else {
                SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 8, !(time(NULL) & 1));    // Blink
            }
            
            break;
        }

        case APP_STATE_DISCONNECT:
        {
            if (bt_l2capResourceManagerRequest(BT_L2CAP_REQUEST_DISCONNECT, appData.link) >= 0) {
                appData.link->reason = HCI_Error_Authentication_Failure;
                appData.state        = APP_STATE_IDLE;
            }
            break;
        }

        case APP_STATE_NO_POWER:
        {
            // Action
            if (appData.tm != time(NULL)) {
                appData.tm =  time(NULL);
                if (appData.powerTimer++ > POWER_OFF_TIME) {
                    appData.state = APP_STATE_SLEEP;
                }
            }

            // Motor control
            r = MOTOR_Tasks(MOTOR_ACTION_NONE);
            switch(r){
            case MOTOR_STATUS_INACTIVE:
                break;
            case MOTOR_STATUS_BUSY:
                break;
            case MOTOR_STATUS_STABLE:
                MOTOR_Tasks(MOTOR_ACTION_FREE);
                break;
            case MOTOR_STATUS_OVERLOAD:
                MOTOR_Tasks(MOTOR_ACTION_FREE);
                break;
            default:
                break;
            }

            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, 1);   // LED A(Pairing)     off
            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 8, 0);   // LED B(Lock status) on

            break;
        }

        case APP_STATE_SLEEP:
        {
            volatile int    i;

            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, 1);   // LED off
            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 8, 1);   // LED off
            SYS_PORTS_ChangeNotificationGlobalEnable(PORTS_ID_0);
            SYS_DEVCON_PowerModeEnter(SYS_POWER_MODE_SLEEP);        // Zzz...
            // Wakeup
            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, 0);   // LED A(Pairing)     on
            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 8, 0);   // LED B(Lock status) on
            for(i = 0; i < 1000000; i++);                           // Starting a bluetooth dongle
            appData.powerTimer = 0;
            appData.state = APP_STATE_WAIT_POWER_STABLE;
            break;
        }

        case APP_STATE_WAIT_POWER_STABLE:
        {
            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 7, 0);   // LED A(Pairing)     on
            SYS_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, 8, 1);   // LED B(Lock status) off
            if (appData.tm != time(NULL)) {
                appData.tm =  time(NULL);
                if (appData.powerTimer++ > POWER_ON_TIME) {
//                  appData.state = APP_STATE_INIT;
                    appData.state = APP_STATE_IDLE;
                }
            }
            break;
        }

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}
 

/*******************************************************************************
 End of File
 */
