//
//  bt_att_data_ALD.c
//
//  Bluetooth Protocol Stack - Attribute Database - Auto Lock Door
//  Copyright(C)2017 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2017.02.13 v0.1  First
//
//

#include "bt_att_db.h"
#include "bt_spec_gatt.h"
#include "bt_spec_att.h"
#include "bt_config.h"

// Alias
#define UINT16(a)  ATT_UINT16_DATA(a)       // Alias (Short name)
#define UINT32(a)  ATT_UINT32_DATA(a)       // Alias
#define UU16(a)    ATT_UUID16TO128_DATA(a)  // Alias
#define UU128(a)   ATT_UUID128_DATA(a)      // Alias

//
//  ATT Database
//
struct bt_attDb bt_attDb = {
    18,
    {// Handle  Type                                                                          Len  Values                                                                       Property
        {0x0001,{UU16(GATT_attribute_gatt_primary_service_declaration)},                       2,  {UINT16(GATT_Services_Generic_Access_Profile)},                              ATT_DB_PROPERTIES_READ},
        {0x0002,    {UU16(GATT_attribute_gatt_characteristic_declaration)},                    5,  {GATT_Characteristic_Properties_Read,
                                                                                                    UINT16(0x0003), UINT16(GATT_characteristic_gap_device_name)},               ATT_DB_PROPERTIES_READ},
        {0x0003,        {UU16(GATT_characteristic_gap_device_name)},                          10,  {'A','L','D',' ','0','.','1',' ','#','1'},                                   ATT_DB_PROPERTIES_READ},
        {0x0004,    {UU16(GATT_attribute_gatt_characteristic_declaration)},                    5,  {GATT_Characteristic_Properties_Read,
                                                                                                    UINT16(0x0005), UINT16(GATT_characteristic_gap_appearance)},                ATT_DB_PROPERTIES_READ},
        {0x0005,        {UU16(GATT_characteristic_gap_appearance)},                            2,  {UINT16(GATT_Characteristic_Appearance_Category_Generic_Remote_Control)},    ATT_DB_PROPERTIES_READ},

        {0x0011,{UU16(GATT_attribute_gatt_primary_service_declaration)},                       2,  {UINT16(GATT_Services_Generic_Attribute_Profile)},                           ATT_DB_PROPERTIES_READ},
        {0x0012,    {UU16(GATT_attribute_gatt_characteristic_declaration)},                    5,  {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write_Without_Response | GATT_Characteristic_Properties_Indicate,
                                                                                                    UINT16(0x0013), UINT16(GATT_characteristic_gatt_service_changed)},          ATT_DB_PROPERTIES_READ},
        {0x0013,        {UU16(GATT_characteristic_gatt_service_changed)},                      4,  {UINT16(0x0001), UINT16(0xffff)},                                            ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x0014,        {UU16(GATT_descriptor_gatt_characteristic_extended_properties)},       2,  {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},      ATT_DB_PROPERTIES_READ},
        {0x0015,        {UU16(GATT_descriptor_gatt_client_characteristic_configuration)},      2,  {UINT16(0x0000)},                                                            ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},

        {0x1000,{UU16(GATT_attribute_gatt_primary_service_declaration)},                       2,  {UINT16(GATT_services_automation_io)},                                       ATT_DB_PROPERTIES_READ},
        {0x1001,    {UU16(GATT_attribute_gatt_characteristic_declaration)},                    5,  {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify,
                                                                                                   UINT16(0x1002), UINT16(GATT_characteristic_digital)},                        ATT_DB_PROPERTIES_READ},
        {0x1002,        {UU16(GATT_characteristic_digital)},                                   1,  {0},                                                                         ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE | ATT_DB_PROPERTIES_AUTHENTICATION},
        {0x1003,        {UU16(GATT_descriptor_gatt_characteristic_extended_properties)},       2,  {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},      ATT_DB_PROPERTIES_READ},
        {0x1004,        {UU16(GATT_descriptor_gatt_client_characteristic_configuration)},      2,  {UINT16(0x0000)},                                                            ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1005,        {UU16(GATT_descriptor_gatt_characteristic_user_description)},         10,  {'L','o','c','k',' ','S','t','a','t','e'},                                   ATT_DB_PROPERTIES_READ},
        {0x1006,        {UU16(GATT_descriptor_gatt_characteristic_presentation_format)},       7,  {GATT_Characteristic_Format_struct, 0,
                                                                                                    UINT16(GATT_Characteristic_Unit_unitless),
                                                                                                    GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers,
                                                                                                    UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},              ATT_DB_PROPERTIES_READ},
        {0x1007,        {UU16(GATT_descriptor_number_of_digitals)},                            1,  {1},                                                                         ATT_DB_PROPERTIES_READ},
    }
};
