//
//  music_box.h
//
//  Beep or music player
//  Copyright(C)2016 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2016.01.19 v0.0  First cording
//
//
#ifndef MUSIC_BOX_H_
#define MUSIC_BOX_H_
#include "system_config.h"          // SYS_CLK_BUS_PERIPHERAL_1

//
//  Config
//
#define MBOX_TMR    TMR_ID_3        // 2 or 3
#define MBOX_OC     OC_ID_3         //
#define MBOX_OC_TMR OC_TIMER_16BIT_TMR3

//
//  Music score data
//
struct mboxScore {                  // 
    short           pr;             // Period count [Peripheral clock]
    unsigned char   onLen;          // Sound on time [tempo]
    unsigned char   offLen;         // No sound time [tempo]
};

//
//  Constant value
//
#define MBOX_FREQ2PR(f) (int)(SYS_CLK_BUS_PERIPHERAL_1 / 2 / (f))
#define MBOX_SCALE_C1   MBOX_FREQ2PR(261.6)
#define MBOX_SCALE_CS1  MBOX_FREQ2PR(277.2)
#define MBOX_SCALE_D1   MBOX_FREQ2PR(293.7)
#define MBOX_SCALE_DS1  MBOX_FREQ2PR(311.1)
#define MBOX_SCALE_E1   MBOX_FREQ2PR(329.6)
#define MBOX_SCALE_F1   MBOX_FREQ2PR(349.2)
#define MBOX_SCALE_FS1  MBOX_FREQ2PR(370.0)
#define MBOX_SCALE_G1   MBOX_FREQ2PR(392.0)
#define MBOX_SCALE_GS1  MBOX_FREQ2PR(415.3)
#define MBOX_SCALE_A1   MBOX_FREQ2PR(440.0)
#define MBOX_SCALE_AS1  MBOX_FREQ2PR(466.1)
#define MBOX_SCALE_B1   MBOX_FREQ2PR(493.9)
#define MBOX_SCALE_C2   MBOX_FREQ2PR(523.2)
#define MBOX_SCALE_CS2  MBOX_FREQ2PR(554.4)
#define MBOX_SCALE_D2   MBOX_FREQ2PR(587.4)
#define MBOX_SCALE_DS2  MBOX_FREQ2PR(622.2)
#define MBOX_SCALE_E2   MBOX_FREQ2PR(659.2)
#define MBOX_SCALE_F2   MBOX_FREQ2PR(698.4)
#define MBOX_SCALE_FS2  MBOX_FREQ2PR(740.0)
#define MBOX_SCALE_G2   MBOX_FREQ2PR(784.0)
#define MBOX_SCALE_GS2  MBOX_FREQ2PR(830.6)
#define MBOX_SCALE_A2   MBOX_FREQ2PR(880.0)
#define MBOX_SCALE_AS2  MBOX_FREQ2PR(932.2)
#define MBOX_SCALE_B2   MBOX_FREQ2PR(987.8)
#define MBOX_SCALE_C3   MBOX_FREQ2PR(1046.4)
#define MBOX_SCALE_CS3  MBOX_FREQ2PR(1108.8)
#define MBOX_SCALE_D3   MBOX_FREQ2PR(1174.8)
#define MBOX_SCALE_DS3  MBOX_FREQ2PR(1244.4)
#define MBOX_SCALE_E3   MBOX_FREQ2PR(1318.4)
#define MBOX_SCALE_F3   MBOX_FREQ2PR(1396.8)
#define MBOX_SCALE_FS3  MBOX_FREQ2PR(1480.0)
#define MBOX_SCALE_G3   MBOX_FREQ2PR(1568.0)
#define MBOX_SCALE_GS3  MBOX_FREQ2PR(1661.2)
#define MBOX_SCALE_A3   MBOX_FREQ2PR(1760.0)
#define MBOX_SCALE_AS3  MBOX_FREQ2PR(1864.4)
#define MBOX_SCALE_B3   MBOX_FREQ2PR(1975.6)
#define MBOX_SCALE_C4   MBOX_FREQ2PR(2092.8)
#define MBOX_SCALE_CS4  MBOX_FREQ2PR(2217.6)
#define MBOX_SCALE_D4   MBOX_FREQ2PR(2349.6)
#define MBOX_SCALE_DS4  MBOX_FREQ2PR(2488.8)
#define MBOX_SCALE_E4   MBOX_FREQ2PR(2636.8)
#define MBOX_SCALE_F4   MBOX_FREQ2PR(2793.6)
#define MBOX_SCALE_FS4  MBOX_FREQ2PR(2960.0)
#define MBOX_SCALE_G4   MBOX_FREQ2PR(3136.0)
#define MBOX_SCALE_GS4  MBOX_FREQ2PR(3322.4)
#define MBOX_SCALE_A4   MBOX_FREQ2PR(3520.0)
#define MBOX_SCALE_AS4  MBOX_FREQ2PR(3728.8)
#define MBOX_SCALE_B4   MBOX_FREQ2PR(3951.2)

//
//  API
//
extern void MBOX_Init(void);
extern void MBOX_Start(const struct mboxScore *score, int tempo, int repeat);
extern void MBOX_Stop(int now);
#define MBOX_Tasks()
// tempo  : onLen,offLen unit[ms]
// repeat : >0 inteval time[ms], 0=1play
// now    : 1=Just stop, 0=score end

//
//  Sample beep data
//
extern const struct mboxScore  MBOX_Beep[];
extern const struct mboxScore  MBOX_BeepUp[];
extern const struct mboxScore  MBOX_BeepDown[];


#endif
